@@ -1,10 +1,9 @@
 /*
 	fbmngplay - fb console MNG player.
-	(c) 2001 by Stefan Reinauer, <stepan@suse.de>
-   
+	(c) 2001 by Stefan Reinauer, <stepan :at: suse.de>
+
 	This program is based on mngplay, part of libmng, written and (C) by
-	
-	Ralph Giles <giles@ashlu.bc.ca>
+	Ralph Giles <giles :at: ashlu.bc.ca>
 
 	This program my be redistributed under the terms of the
 	GNU General Public Licence, version 2, or at your preference,
