@@ -4,8 +4,8 @@
 /* ************************************************************************** */
 /* *                                                                        * */
 /* * project   : libmng                                                     * */
-/* * file      : libmng_types.h            copyright (c) 2000-2004 G.Juyn   * */
-/* * version   : 1.0.8                                                      * */
+/* * file      : libmng_types.h            copyright (c) 2000-2007 G.Juyn   * */
+/* * version   : 1.0.10                                                     * */
 /* *                                                                        * */
 /* * purpose   : type specifications                                        * */
 /* *                                                                        * */
@@ -110,11 +110,22 @@
 /* *                                                                        * */
 /* *             1.0.7 - 03/10/2004 - G.R-P                                 * */
 /* *             - added conditionals around openstream/closestream         * */
+/* *                                                                        * */
 /* *             1.0.8 - 04/11/2004 - G.Juyn                                * */
 /* *             - added data-push mechanisms for specialized decoders      * */
 /* *             1.0.8 - 08/01/2004 - G.Juyn                                * */
 /* *             - added support for 3+byte pixelsize for JPEG's            * */
 /* *                                                                        * */
+/* *             1.0.9 - 12/05/2004 - G.Juyn                                * */
+/* *             - inclusion of zlib/lcms/ijgsrc6b with <> instead of ""    * */
+/* *             1.0.9 - 12/06/2004 - G.Juyn                                * */
+/* *             - added conditional MNG_OPTIMIZE_CHUNKREADER               * */
+/* *                                                                        * */
+/* *             1.0.10 - 04/08/2007 - G.Juyn                               * */
+/* *             - added support for mPNG proposal                          * */
+/* *             1.0.10 - 04/12/2007 - G.Juyn                               * */
+/* *             - added support for ANG proposal                           * */
+/* *                                                                        * */
 /* ************************************************************************** */
 
 #ifndef _libmng_types_h_
@@ -140,12 +151,13 @@
 /* *  together)                                                             * */
 /* *                                                                        * */
 /* ************************************************************************** */
-
+#ifdef HIDDEN_FOR_NOW
 #ifdef WIN32                           /* only include needed stuff */
 #ifndef WIN32_LEAN_AND_MEAN
 #define WIN32_LEAN_AND_MEAN
 #endif
 #endif
+#endif /* HIDDEN_FOR_NOW */
 
 #ifdef MNG_USE_DLL
 #ifdef MNG_SKIP_ZLIB
@@ -160,30 +172,39 @@
 #endif
 
 #ifdef MNG_INCLUDE_ZLIB                /* zlib by Mark Adler & Jean-loup Gailly */
-#include "zlib.h"
+#include <zlib.h>
 #endif
 
 #ifdef MNG_INCLUDE_LCMS                /* little cms by Marti Maria Saguer */
 #ifndef ZLIB_DLL
 #undef FAR
 #endif
-#include "lcms.h"
+#ifdef HAVE_LIBLCMS1
+#include <lcms.h>
+#elif defined(HAVE_LIBLCMS2)
+#include <lcms2.h>
+#endif
 #endif /* MNG_INCLUDE_LCMS */
 
 #ifdef MNG_INCLUDE_IJG6B               /* IJG's jpgsrc6b */
-#include <stdio.h>
+#define JPEG_INTERNAL_OPTIONS          /* for RGB_PIXELSIZE */
 #ifdef MNG_USE_SETJMP
 #include <setjmp.h>                    /* needed for error-recovery (blergh) */
 #else
 #ifdef WIN32
-#define USE_WINDOWS_MESSAGEBOX         /* display a messagebox under Windoze */
+/* #define USE_WINDOWS_MESSAGEBOX */        /* display a messagebox under Windoze */
 #endif
 #endif /* MNG_USE_SETJMP */
 #ifdef FAR
 #undef FAR                             /* possibly defined by zlib or lcms */
 #endif
 #define JPEG_INTERNAL_OPTIONS          /* for RGB_PIXELSIZE */
-#include "jpeglib.h"                   /* all that for JPEG support  :-) */
+/* There has been a change in jpeg-9 : */
+#ifndef _WIN32
+#define HAVE_BOOLEAN
+typedef int boolean;
+#endif
+#include <jpeglib.h>
 #endif /* MNG_INCLUDE_IJG6B */
 
 #if defined(MNG_INTERNAL_MEMMNGMT) || defined(MNG_INCLUDE_FILTERS)
@@ -377,19 +398,36 @@ typedef cmsHPROFILE         mng_cmsprof;         /* little CMS helper defs */
 typedef cmsHTRANSFORM       mng_cmstrans;
 typedef cmsCIExyY           mng_CIExyY;
 typedef cmsCIExyYTRIPLE     mng_CIExyYTRIPLE;
+
+#ifdef HAVE_LIBLCMS1
 typedef LPGAMMATABLE        mng_gammatabp;
+#elif defined(HAVE_LIBLCMS2)
+typedef cmsToneCurve*       mng_gammatabp;
+#endif
 #endif /* MNG_INCLUDE_LCMS */
 
 /* ************************************************************************** */
 
                                        /* enumeration of known graphics types */
-enum mng_imgtypes {mng_it_unknown, mng_it_png, mng_it_mng, mng_it_jng};
+enum mng_imgtypes {mng_it_unknown, mng_it_png, mng_it_mng, mng_it_jng
+#ifdef MNG_INCLUDE_MPNG_PROPOSAL
+     ,mng_it_mpng
+#endif     
+#ifdef MNG_INCLUDE_ANG_PROPOSAL
+     ,mng_it_ang
+#endif
+     };
 typedef enum mng_imgtypes mng_imgtype;
 
                                        /* enumeration of animation speed-types */
 enum mng_speedtypes {mng_st_normal, mng_st_fast, mng_st_slow, mng_st_slowest};
 typedef enum mng_speedtypes mng_speedtype;
 
+#ifdef MNG_OPTIMIZE_CHUNKREADER
+                                       /* enumeration object-creation indicators */
+enum mng_createobjtypes {mng_create_none, mng_create_always, mng_create_ifglobal};
+typedef enum mng_createobjtypes mng_createobjtype;
+#endif
 
 /* ************************************************************************** */
 
