@@ -1,17 +1,16 @@
+#include "config.h"
 /* ************************************************************************** */
 /* *             For conditions of distribution and use,                    * */
 /* *                see copyright notice in libmng.h                        * */
 /* ************************************************************************** */
 /* *                                                                        * */
 /* * project   : libmng                                                     * */
-/* * file      : libmng_zlib.c             copyright (c) 2000-2002 G.Juyn   * */
-/* * version   : 1.0.5                                                      * */
+/* * file      : libmng_zlib.c             copyright (c) 2000-2004 G.Juyn   * */
+/* * version   : 1.0.9                                                      * */
 /* *                                                                        * */
 /* * purpose   : ZLIB library interface (implementation)                    * */
 /* *                                                                        * */
 /* * author    : G.Juyn                                                     * */
-/* * web       : http://www.3-t.com                                         * */
-/* * email     : mailto:info@3-t.com                                        * */
 /* *                                                                        * */
 /* * comment   : implementation of the ZLIB library interface               * */
 /* *                                                                        * */
@@ -46,6 +45,12 @@
 /* *             1.0.5 - 09/19/2002 - G.Juyn                                * */
 /* *             - added warning for too much IDAT data                     * */
 /* *                                                                        * */
+/* *             1.0.6 - 07/07/2003 - G.R-P                                 * */
+/* *             - added MNG_NO_16BIT_SUPPORT support                       * */
+/* *                                                                        * */
+/* *             1.0.9 - 10/09/2004 - G.R-P                                 * */
+/* *             - added MNG_NO_1_2_4BIT_SUPPORT support                    * */
+/* *                                                                        * */
 /* ************************************************************************** */
 
 #include "libmng.h"
@@ -106,7 +111,7 @@ void mngzlib_free (voidpf pData,
 mng_retcode mngzlib_initialize (mng_datap pData)
 {
 #ifdef MNG_SUPPORT_TRACE
-  MNG_TRACE (pData, MNG_FN_ZLIB_INITIALIZE, MNG_LC_START)
+  MNG_TRACE (pData, MNG_FN_ZLIB_INITIALIZE, MNG_LC_START);
 #endif
 
 #ifdef MNG_INTERNAL_MEMMNGMT
@@ -123,7 +128,7 @@ mng_retcode mngzlib_initialize (mng_datap pData)
   pData->bDeflating   = MNG_FALSE;
 
 #ifdef MNG_SUPPORT_TRACE
-  MNG_TRACE (pData, MNG_FN_ZLIB_INITIALIZE, MNG_LC_END)
+  MNG_TRACE (pData, MNG_FN_ZLIB_INITIALIZE, MNG_LC_END);
 #endif
 
   return MNG_NOERROR;                  /* done */
@@ -134,7 +139,7 @@ mng_retcode mngzlib_initialize (mng_datap pData)
 mng_retcode mngzlib_cleanup (mng_datap pData)
 {
 #ifdef MNG_SUPPORT_TRACE
-  MNG_TRACE (pData, MNG_FN_ZLIB_CLEANUP, MNG_LC_START)
+  MNG_TRACE (pData, MNG_FN_ZLIB_CLEANUP, MNG_LC_START);
 #endif
 
   if (pData->bInflating)               /* force zlib cleanup */
@@ -143,7 +148,7 @@ mng_retcode mngzlib_cleanup (mng_datap pData)
     mngzlib_deflatefree (pData);
 
 #ifdef MNG_SUPPORT_TRACE
-  MNG_TRACE (pData, MNG_FN_ZLIB_CLEANUP, MNG_LC_END)
+  MNG_TRACE (pData, MNG_FN_ZLIB_CLEANUP, MNG_LC_END);
 #endif
 
   return MNG_NOERROR;                  /* done */
@@ -156,19 +161,19 @@ mng_retcode mngzlib_inflateinit (mng_datap pData)
   int iZrslt;
 
 #ifdef MNG_SUPPORT_TRACE
-  MNG_TRACE (pData, MNG_FN_ZLIB_INFLATEINIT, MNG_LC_START)
+  MNG_TRACE (pData, MNG_FN_ZLIB_INFLATEINIT, MNG_LC_START);
 #endif
                                        /* initialize zlib structures and such */
   iZrslt = inflateInit (&pData->sZlib);
 
   if (iZrslt != Z_OK)                  /* on error bail out */
-    MNG_ERRORZ (pData, (mng_uint32)iZrslt)
+    MNG_ERRORZ (pData, (mng_uint32)iZrslt);
 
   pData->bInflating      = MNG_TRUE;   /* really inflating something now */
   pData->sZlib.next_out  = 0;          /* force JIT initialization */
 
 #ifdef MNG_SUPPORT_TRACE
-  MNG_TRACE (pData, MNG_FN_ZLIB_INFLATEINIT, MNG_LC_END)
+  MNG_TRACE (pData, MNG_FN_ZLIB_INFLATEINIT, MNG_LC_END);
 #endif
 
   return MNG_NOERROR;                  /* done */
@@ -186,7 +191,7 @@ mng_retcode mngzlib_inflaterows (mng_datap  pData,
   mng_ptr     pSwap;
 
 #ifdef MNG_SUPPORT_TRACE
-  MNG_TRACE (pData, MNG_FN_ZLIB_INFLATEROWS, MNG_LC_START)
+  MNG_TRACE (pData, MNG_FN_ZLIB_INFLATEROWS, MNG_LC_START);
 #endif
 
   pData->sZlib.next_in   = pIndata;    /* let zlib know where to get stuff */
@@ -196,6 +201,15 @@ mng_retcode mngzlib_inflaterows (mng_datap  pData,
   {                                    /* let zlib know where to store stuff */
     pData->sZlib.next_out  = pData->pWorkrow;
     pData->sZlib.avail_out = (uInt)(pData->iRowsize + pData->iPixelofs);
+#ifdef MNG_NO_1_2_4BIT_SUPPORT
+    if (pData->iPNGdepth < 8)
+       pData->sZlib.avail_out = (uInt)((pData->iPNGdepth*pData->iRowsize + 7)/8
+           + pData->iPixelofs);
+#endif
+#ifdef MNG_NO_16BIT_SUPPORT
+    if (pData->iPNGdepth > 8)
+       pData->sZlib.avail_out = (uInt)(2*pData->iRowsize + pData->iPixelofs);
+#endif
   }
 
   do
@@ -207,6 +221,128 @@ mng_retcode mngzlib_inflaterows (mng_datap  pData,
     {                                  /* image not completed yet ? */
       if (pData->iRow < (mng_int32)pData->iDataheight)
       {
+#ifdef MNG_NO_1_2_4BIT_SUPPORT
+        if (pData->iPNGdepth == 1)
+        {
+          /* Inflate Workrow to 8-bit */
+          mng_int32  iX;
+          mng_uint8p pSrc = pData->pWorkrow+1;
+          mng_uint8p pDest = pSrc + pData->iRowsize - (pData->iRowsize+7)/8;
+
+          for (iX = ((pData->iRowsize+7)/8) ; iX > 0 ; iX--)
+             *pDest++ = *pSrc++;
+
+          pDest = pData->pWorkrow+1;
+          pSrc = pDest + pData->iRowsize - (pData->iRowsize+7)/8;
+          for (iX = pData->iRowsize; ;)
+          {
+            *pDest++ = (((*pSrc)>>7)&1);
+            if (iX-- <= 0)
+              break;
+            *pDest++ = (((*pSrc)>>6)&1);
+            if (iX-- <= 0)
+              break;
+            *pDest++ = (((*pSrc)>>5)&1);
+            if (iX-- <= 0)
+              break;
+            *pDest++ = (((*pSrc)>>4)&1);
+            if (iX-- <= 0)
+              break;
+            *pDest++ = (((*pSrc)>>3)&1);
+            if (iX-- <= 0)
+              break;
+            *pDest++ = (((*pSrc)>>2)&1);
+            if (iX-- <= 0)
+              break;
+            *pDest++ = (((*pSrc)>>1)&1);
+            if (iX-- <= 0)
+              break;
+            *pDest++ = (((*pSrc)   )&1);
+            if (iX-- <= 0)
+              break;
+            pSrc++;
+          }
+        }
+        else if (pData->iPNGdepth == 2)
+        {
+          /* Inflate Workrow to 8-bit */
+          mng_int32  iX;
+          mng_uint8p pSrc = pData->pWorkrow+1;
+          mng_uint8p pDest = pSrc + pData->iRowsize - (2*pData->iRowsize+7)/8;
+
+          for (iX = ((2*pData->iRowsize+7)/8) ; iX > 0 ; iX--)
+             *pDest++ = *pSrc++;
+
+          pDest = pData->pWorkrow+1;
+          pSrc = pDest + pData->iRowsize - (2*pData->iRowsize+7)/8;
+          for (iX = pData->iRowsize; ;)
+          {
+            *pDest++ = (((*pSrc)>>6)&3);
+            if (iX-- <= 0)
+              break;
+            *pDest++ = (((*pSrc)>>4)&3);
+            if (iX-- <= 0)
+              break;
+            *pDest++ = (((*pSrc)>>2)&3);
+            if (iX-- <= 0)
+              break;
+            *pDest++ = (((*pSrc)   )&3);
+            if (iX-- <= 0)
+              break;
+            pSrc++;
+          }
+        }
+        else if (pData->iPNGdepth == 4)
+        {
+          /* Inflate Workrow to 8-bit */
+          mng_int32  iX;
+          mng_uint8p pSrc = pData->pWorkrow+1;
+          mng_uint8p pDest = pSrc + pData->iRowsize - (4*pData->iRowsize+7)/8;
+
+          for (iX = ((4*pData->iRowsize+7)/8) ; iX > 0 ; iX--)
+             *pDest++ = *pSrc++;
+
+          pDest = pData->pWorkrow+1;
+          pSrc = pDest + pData->iRowsize - (4*pData->iRowsize+7)/8;
+          for (iX = pData->iRowsize; ;)
+          {
+            *pDest++ = (((*pSrc)>>4)&0x0f);
+            if (iX-- <= 0)
+              break;
+            *pDest++ = (((*pSrc)   )&0x0f);
+            if (iX-- <= 0)
+              break;
+            pSrc++;
+          }
+        }
+        if (pData->iPNGdepth < 8 && pData->iColortype == 0)
+        {
+          /* Expand samples to 8-bit by LBR */
+          mng_int32  iX;
+          mng_uint8p pSrc = pData->pWorkrow+1;
+          mng_uint8 multiplier[]={0,255,85,0,17,0,0,0,1};
+
+          for (iX = pData->iRowsize; iX > 0; iX--)
+              *pSrc++ *= multiplier[pData->iPNGdepth];
+        }
+#endif
+#ifdef MNG_NO_16BIT_SUPPORT
+        if (pData->iPNGdepth > 8)
+        {
+          /* Reduce Workrow to 8-bit */
+          mng_int32  iX;
+          mng_uint8p pSrc = pData->pWorkrow+1;
+          mng_uint8p pDest = pSrc;
+
+          for (iX = pData->iRowsize; iX > 0; iX--)
+          {
+            *pDest = *pSrc;
+            pDest++;
+            pSrc+=2;
+          }
+        }
+#endif
+
 #ifdef FILTER192                       /* has leveling info ? */
         if (pData->iFilterofs == MNG_FILTER_DIFFERING)
           iRslt = init_rowdiffering (pData);
@@ -219,7 +355,7 @@ mng_retcode mngzlib_inflaterows (mng_datap  pData,
           iRslt = mng_filter_a_row (pData);
         else
           iRslt = MNG_NOERROR;
-                                       /* additonal leveling/differing ? */
+                                       /* additional leveling/differing ? */
         if ((!iRslt) && (pData->fDifferrow))
         {
           iRslt = ((mng_differrow)pData->fDifferrow) (pData);
@@ -278,6 +414,15 @@ mng_retcode mngzlib_inflaterows (mng_datap  pData,
                                        /* let zlib know where to store next output */
       pData->sZlib.next_out  = pData->pWorkrow;
       pData->sZlib.avail_out = (uInt)(pData->iRowsize + pData->iPixelofs);
+#ifdef MNG_NO_1_2_4BIT_SUPPORT
+    if (pData->iPNGdepth < 8)
+       pData->sZlib.avail_out = (uInt)((pData->iPNGdepth*pData->iRowsize + 7)/8
+           + pData->iPixelofs);
+#endif
+#ifdef MNG_NO_16BIT_SUPPORT
+      if (pData->iPNGdepth > 8)
+        pData->sZlib.avail_out = (uInt)(2*pData->iRowsize + pData->iPixelofs);
+#endif
     }
   }                                    /* until some error or EOI
                                           or all pixels received */
@@ -286,13 +431,13 @@ mng_retcode mngzlib_inflaterows (mng_datap  pData,
             ( (pData->iPass >= 0) && (pData->iPass < 7) )    )    );
                                        /* on error bail out */
   if ((iZrslt != Z_OK) && (iZrslt != Z_STREAM_END))
-    MNG_ERRORZ (pData, (mng_uint32)iZrslt)
+    MNG_ERRORZ (pData, (mng_uint32)iZrslt);
                                        /* too much data ? */
   if ((iZrslt == Z_OK) && (pData->sZlib.avail_in > 0))
-    MNG_WARNING (pData, MNG_TOOMUCHIDAT)
+    MNG_WARNING (pData, MNG_TOOMUCHIDAT);
 
 #ifdef MNG_SUPPORT_TRACE
-  MNG_TRACE (pData, MNG_FN_ZLIB_INFLATEROWS, MNG_LC_END)
+  MNG_TRACE (pData, MNG_FN_ZLIB_INFLATEROWS, MNG_LC_END);
 #endif
 
   return MNG_NOERROR;
@@ -308,7 +453,7 @@ mng_retcode mngzlib_inflatedata (mng_datap  pData,
   int iZrslt;
 
 #ifdef MNG_SUPPORT_TRACE
-  MNG_TRACE (pData, MNG_FN_ZLIB_INFLATEDATA, MNG_LC_START)
+  MNG_TRACE (pData, MNG_FN_ZLIB_INFLATEDATA, MNG_LC_START);
 #endif
                                        /* let zlib know where to get stuff */
   pData->sZlib.next_in   = pIndata;
@@ -320,10 +465,10 @@ mng_retcode mngzlib_inflatedata (mng_datap  pData,
     return MNG_BUFOVERFLOW;
                                        /* on error bail out */
   if ((iZrslt != Z_OK) && (iZrslt != Z_STREAM_END))
-    MNG_ERRORZ (pData, (mng_uint32)iZrslt)
+    MNG_ERRORZ (pData, (mng_uint32)iZrslt);
 
 #ifdef MNG_SUPPORT_TRACE
-  MNG_TRACE (pData, MNG_FN_ZLIB_INFLATEDATA, MNG_LC_END)
+  MNG_TRACE (pData, MNG_FN_ZLIB_INFLATEDATA, MNG_LC_END);
 #endif
 
   return MNG_NOERROR;
@@ -336,18 +481,18 @@ mng_retcode mngzlib_inflatefree (mng_datap pData)
   int iZrslt;
 
 #ifdef MNG_SUPPORT_TRACE
-  MNG_TRACE (pData, MNG_FN_ZLIB_INFLATEFREE, MNG_LC_START)
+  MNG_TRACE (pData, MNG_FN_ZLIB_INFLATEFREE, MNG_LC_START);
 #endif
 
   pData->bInflating = MNG_FALSE;       /* stopped it */
 
-  iZrslt = inflateEnd (&pData->sZlib); /* let zlib cleanup it's own stuff */
+  iZrslt = inflateEnd (&pData->sZlib); /* let zlib cleanup its own stuff */
 
   if (iZrslt != Z_OK)                  /* on error bail out */
-    MNG_ERRORZ (pData, (mng_uint32)iZrslt)
+    MNG_ERRORZ (pData, (mng_uint32)iZrslt);
 
 #ifdef MNG_SUPPORT_TRACE
-  MNG_TRACE (pData, MNG_FN_ZLIB_INFLATEFREE, MNG_LC_END)
+  MNG_TRACE (pData, MNG_FN_ZLIB_INFLATEFREE, MNG_LC_END);
 #endif
 
   return MNG_NOERROR;                  /* done */
@@ -360,7 +505,7 @@ mng_retcode mngzlib_deflateinit (mng_datap pData)
   int iZrslt;
 
 #ifdef MNG_SUPPORT_TRACE
-  MNG_TRACE (pData, MNG_FN_ZLIB_DEFLATEINIT, MNG_LC_START)
+  MNG_TRACE (pData, MNG_FN_ZLIB_DEFLATEINIT, MNG_LC_START);
 #endif
                                        /* initialize zlib structures and such */
   iZrslt = deflateInit2 (&pData->sZlib, pData->iZlevel, pData->iZmethod,
@@ -368,12 +513,12 @@ mng_retcode mngzlib_deflateinit (mng_datap pData)
                          pData->iZstrategy);
 
   if (iZrslt != Z_OK)                  /* on error bail out */
-    MNG_ERRORZ (pData, (mng_uint32)iZrslt)
+    MNG_ERRORZ (pData, (mng_uint32)iZrslt);
 
   pData->bDeflating = MNG_TRUE;        /* really deflating something now */
 
 #ifdef MNG_SUPPORT_TRACE
-  MNG_TRACE (pData, MNG_FN_ZLIB_DEFLATEINIT, MNG_LC_END)
+  MNG_TRACE (pData, MNG_FN_ZLIB_DEFLATEINIT, MNG_LC_END);
 #endif
 
   return MNG_NOERROR;                  /* done */
@@ -386,14 +531,14 @@ mng_retcode mngzlib_deflaterows (mng_datap  pData,
                                  mng_uint8p pIndata)
 {
 #ifdef MNG_SUPPORT_TRACE
-  MNG_TRACE (pData, MNG_FN_ZLIB_DEFLATEROWS, MNG_LC_START)
+  MNG_TRACE (pData, MNG_FN_ZLIB_DEFLATEROWS, MNG_LC_START);
 #endif
 
 
 
 
 #ifdef MNG_SUPPORT_TRACE
-  MNG_TRACE (pData, MNG_FN_ZLIB_DEFLATEROWS, MNG_LC_END)
+  MNG_TRACE (pData, MNG_FN_ZLIB_DEFLATEROWS, MNG_LC_END);
 #endif
 
   return MNG_NOERROR;
@@ -408,7 +553,7 @@ mng_retcode mngzlib_deflatedata (mng_datap  pData,
   int iZrslt;
 
 #ifdef MNG_SUPPORT_TRACE
-  MNG_TRACE (pData, MNG_FN_ZLIB_DEFLATEDATA, MNG_LC_START)
+  MNG_TRACE (pData, MNG_FN_ZLIB_DEFLATEDATA, MNG_LC_START);
 #endif
 
   pData->sZlib.next_in  = pIndata;     /* let zlib know where to get stuff */
@@ -420,10 +565,10 @@ mng_retcode mngzlib_deflatedata (mng_datap  pData,
     return MNG_BUFOVERFLOW;
                                        /* on error bail out */
   if ((iZrslt != Z_OK) && (iZrslt != Z_STREAM_END))
-    MNG_ERRORZ (pData, (mng_uint32)iZrslt)
+    MNG_ERRORZ (pData, (mng_uint32)iZrslt);
 
 #ifdef MNG_SUPPORT_TRACE
-  MNG_TRACE (pData, MNG_FN_ZLIB_DEFLATEDATA, MNG_LC_END)
+  MNG_TRACE (pData, MNG_FN_ZLIB_DEFLATEDATA, MNG_LC_END);
 #endif
 
   return MNG_NOERROR;
@@ -436,18 +581,18 @@ mng_retcode mngzlib_deflatefree (mng_datap pData)
   int iZrslt;
 
 #ifdef MNG_SUPPORT_TRACE
-  MNG_TRACE (pData, MNG_FN_ZLIB_DEFLATEFREE, MNG_LC_START)
+  MNG_TRACE (pData, MNG_FN_ZLIB_DEFLATEFREE, MNG_LC_START);
 #endif
 
-  iZrslt = deflateEnd (&pData->sZlib); /* let zlib cleanup it's own stuff */
+  iZrslt = deflateEnd (&pData->sZlib); /* let zlib cleanup its own stuff */
 
   if (iZrslt != Z_OK)                  /* on error bail out */
-    MNG_ERRORZ (pData, (mng_uint32)iZrslt)
+    MNG_ERRORZ (pData, (mng_uint32)iZrslt);
 
   pData->bDeflating = MNG_FALSE;       /* stopped it */
 
 #ifdef MNG_SUPPORT_TRACE
-  MNG_TRACE (pData, MNG_FN_ZLIB_DEFLATEFREE, MNG_LC_END)
+  MNG_TRACE (pData, MNG_FN_ZLIB_DEFLATEFREE, MNG_LC_END);
 #endif
 
   return MNG_NOERROR;                  /* done */
